package cz.autocont.iszr.demo;

import jakarta.xml.ws.AsyncHandler;
import jakarta.xml.ws.Response;
import java.util.concurrent.ExecutionException;

/**
 * Demonstrace asynchronniho volani webove sluzby.
 * Vysledek volani je ziskany pomoci call-back metody.<br/>
 * <i>Dalsim zpusoben asychronniho volani je "aktivni cekani".</i>
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @param <REQ> request, ktery se odesila pri volani webove sluzby
 * @param <RES> response, ktery vrati webova sluzba
 * @author Vladimir Pach
 * @since 0.0.1
 */
public abstract class AsyncDemoCall<REQ, RES> extends BaseDemoCall<REQ, RES> implements AsyncHandler<RES>
{

    /**
     * Demo volani webove sluzby.
     * 
     * @throws DemoException Chyba při volání služby
     * @see #createRequestData();
     */
    @Override
    public void demoCallService() throws DemoException
    {
        // prepare request data
        REQ requestData = createRequestData();

        // call service
        callService(requestData);
    }

    /**
     * call-back metoda pro zpracovani vysledku volani webove sluzby.
     * Tuto metodu zavola STUB nezavisle na hlavnim vlakne.
     * @see AsyncHandler
     */
    @Override
    public void handleResponse(Response<RES> res)
    {
        try
        {
            processResponseData(res.get());
        } catch (DemoException | InterruptedException | ExecutionException e)
        {
            e.printStackTrace();
        }
    }
}
